program Project2;
{$APPTYPE CONSOLE}
uses
  sysutils,
  Classes,
  TypInfo,
  PAXCOMP_CONSTANTS,
  PaxCompiler,
  PaxCompilerExplorer,
  PaxCompilerDebugger,
  PaxBasicLanguage,
  PaxProgram,
  PaxRegister,
  IMPORT_Common;

type
  TMyHandler = class
  private
    L: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure SaveToDisk;
    function UsedUnitHandler(Sender: TPaxCompiler; const UnitName: String;
                              var SourceCode: String): Boolean;
    function SavePCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
    function LoadPCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
    function LoadPCUProgramHandler(Sender: TPaxProgram; const UnitName: String): TStream;
  end;

constructor TMyHandler.Create;
begin
  inherited;
  L := TStringList.Create;
end;

destructor TMyHandler.Destroy;
var
  I: Integer;
begin
  for I := 0 to L.Count - 1 do
    L.Objects[I].Free;

  L.Free;
  inherited;
end;

procedure TMyHandler.SaveToDisk;
var
  I: Integer;
  Stream: TMemoryStream;
  S: String;
begin
  for I := 0 to L.Count - 1 do
  begin
    S := L[I];
    Stream := TMemoryStream(L.Objects[I]);
    Stream.Position := 0;
    Stream.SaveToFile(S + '.PCU');
  end;
end;

function TMyHandler.UsedUnitHandler(Sender: TPaxCompiler; const UnitName: String;
                          var SourceCode: String): Boolean;
var
  L: TStringList;
begin
  if CompareText(UnitName, 'MyModule') = 0 then
  begin
    L := TStringList.Create;
    try
      L.LoadFromFile('MyModule.txt'); // just to load it from somewhere
      SourceCode := L.Text;
    finally
      L.Free;
    end;
    result := true;
  end;
end;

function TMyHandler.SavePCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
begin
  result := TMemoryStream.Create;
  L.AddObject(UnitName, result);
end;

function TMyHandler.LoadPCUCompilerHandler(Sender: TPaxCompiler; const UnitName: String): TStream;
var
  I: Integer;
begin
  result := nil;
  for I := 0 to L.Count - 1 do
    if CompareText(L[I], UnitName) = 0 then
    begin
      result := TStream(L.Objects[I]);
      result.Position := 0;
      Exit;
    end;
end;

function TMyHandler.LoadPCUProgramHandler(Sender: TPaxProgram; const UnitName: String): TStream;
var
  I: Integer;
begin
  result := nil;
  for I := 0 to L.Count - 1 do
    if CompareText(L[I], UnitName) = 0 then
    begin
      result := TStream(L.Objects[I]);
      result.Position := 0;
      Exit;
    end;
end;

var
  MyHandler: TMyHandler;

var
  PaxCompiler1: TPaxCompiler;
  PaxProgram1: TPaxProgram;
  PaxBasicLanguage1: TPaxBasicLanguage;
begin
  // Build all units, save pcu-files and create compiled script

  MyHandler := TMyHandler.Create;
  try

    PaxCompiler1 := TPaxCompiler.Create(nil);
    PaxCompiler1.OnUsedUnit := MyHandler.UsedUnitHandler;
//    PaxCompiler1.OnSavePCU := MyHandler.SavePCUCompilerHandler;
//    PaxCompiler1.OnLoadPCU := MyHandler.LoadPCUCompilerHandler;

    PaxProgram1 := TPaxProgram.Create(nil);
    PaxBasicLanguage1 := TPaxBasicLanguage.Create(nil);
    try
      PaxCompiler1.RegisterLanguage(PaxBasicLanguage1);
      PaxCompiler1.AddModule('1', 'Basic');
      PaxCompiler1.AddCodeFromFile('1', 'script.txt');

      if PaxCompiler1.Compile(PaxProgram1, true, true) then
      // build with run-time packages
      begin
        PaxProgram1.SaveToFile('script.bin');
//        PaxProgram1.Run;
//        writeln('Press any key...');
//        Readln;
//        Exit;
      end
      else
      begin
        writeln(PaxCompiler1.ErrorMessage[0]);
        writeln(PaxCompiler1.ErrorLine[0]);
        writeln('Press any key...');
        Readln;
        Exit;
      end;
    finally
      PaxCompiler1.Free;
      PaxProgram1.Free;
      PaxBasicLanguage1.Free;
    end;

//    MyHandler.SaveToDisk;

    // Use compiled units (pcu-files) at run-tme as run-time packages

    PaxProgram1 := TPaxProgram.Create(nil);
    try
//    PaxProgram1.OnLoadPCU := MyHandler.LoadPCUProgramHandler;

      PaxProgram1.LoadFromFile('script.bin');
      PaxProgram1.Run;
    finally
      PaxProgram1.Free;
    end;

  finally
    MyHandler.Free;
  end;

  writeln('Press any key...');
  Readln;
end.
